import pathlib

from aiohttp import web

from unv.utils.files import calc_crc32_for_file

from .settings import SETTINGS


async def render_template(
        request, template_name, context=None, context_processors=None,
        status=web.HTTPOk.status_code):
    context = context or {}
    context_processors = context_processors or {}
    template = request.app['jinja2'].get_template(template_name)

    for key, processor in context_processors.items():
        if key not in context:
            value = await processor(request)
            context[key] = value

    text = template.render(context)

    return web.Response(
        text=text, status=status, charset='utf-8', content_type='text/html')


def url_for_media(path):
    path = path.replace(SETTINGS['media']['paths']['public'], '')
    return ''.join([SETTINGS['media']['urls']['public'], path])


def url_for_private_media(path):
    path = path.replace(SETTINGS['media']['paths']['private'], '')
    return ''.join([SETTINGS['media']['urls']['private'], path])


def url_for_static(path):
    real_path = pathlib.Path(SETTINGS['static']['paths']['public'])
    real_path = real_path / path.lstrip('/')
    hash_ = '?hash={}'.format(calc_crc32_for_file(real_path))
    path = path.replace(SETTINGS['static']['paths']['public'], '')
    return ''.join([SETTINGS['static']['urls']['public'], path, hash_])


def url_for_private_static(path):
    real_path = pathlib.Path(SETTINGS['static']['paths']['private'])
    real_path = real_path / path.lstrip('/')
    hash_ = '?hash={}'.format(calc_crc32_for_file(real_path))
    path = path.replace(SETTINGS['static']['paths']['private'], '')
    return ''.join([SETTINGS['static']['urls']['private'], path, hash_])


def url_with_domain(path):
    return '{}://{}{}'.format(SETTINGS['protocol'], SETTINGS['domain'], path)


def make_url_for_func(app):
    def url_for(route, **parts):
        parts = {key: str(value) for key, value in parts.items()}
        return app.router[route].url_for(**parts)
    return url_for


def make_url_with_domain_for_func(app):
    def url_with_domain_for(route, **parts):
        url = app.router[route].url_for(**parts)
        return url_with_domain(str(url))
    return url_with_domain_for


def inline_static_from(path):
    static_path = pathlib.Path(SETTINGS['static']['paths']['public'])
    with open(str(static_path / path.lstrip('/')), 'r') as f:
        return f.read().replace("\n", "")



from setuptools import setup, find_packages

setup(
    name='unv.web',
    version='0.1.1',
    description="""Web application helpers for unv based on aiohttp""",
    url='http://github.com/c137digital/unv_web',
    author='Morty Space',
    author_email='morty.space@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=[
        'aiohttp',
        'uvloop',
        'ujson',
        'unv.app==0.1.3'
    ],
    zip_safe=True,
    # entry_points={
    #     'console_scripts': [
    #         'run = package.app:main',
    #     ]
    # },
)



import functools

import ujson as json

from aiohttp import web


def as_json(f):
    """Return json response from passed dict result."""
    @functools.wraps(f)
    async def wrapper(request, *args, **kwargs):
        data = await f(request, *args, **kwargs)
        return web.json_response(data, dumps=json.dumps)
    return wrapper


def as_json_method(f):
    """Return json response from passed dict result."""
    @functools.wraps(f)
    async def wrapper(self, request, *args, **kwargs):
        data = await f(self, request, *args, **kwargs)
        return web.json_response(data or {}, dumps=json.dumps)
    return wrapper


def cached(f):
    @functools.wraps(f)
    async def wrapper(request, *args, **kwargs):
        key = (f.__name__, request.path)
        if key in request.app['cache']:
            return request.app['cache'][key]
        result = await f(request, *args, **kwargs)
        request.app['cache'][key] = result
        return result
    return wrapper


def render(template_name: str, context_processors: dict = None, status = 200):
    """Render jinja2 template by given name and custom context processors."""
    context_processors = context_processors or {}

    def decorator(f):
        @functools.wraps(f)
        async def wrapper(request, *args, **kwargs):
            context = await f(request, *args, **kwargs)
            template = request.app['jinja2'].get_template(template_name)

            for key, processor in context_processors.items():
                if key not in context:
                    value = await processor(request)
                    context[key] = value

            context['request'] = request
            text = template.render(context)

            return web.Response(
                text=text, status=status, charset='utf-8',
                content_type='text/html'
            )
        return wrapper

    return decorator


def render_method(
        template_name: str, context_processors: dict = None, status = 200):
    """Render jinja2 template by given name and custom context processors."""
    context_processors = context_processors or {}

    def decorator(f):
        @functools.wraps(f)
        async def wrapper(self, request, *args, **kwargs):
            context = await f(self, request, *args, **kwargs)
            template = request.app['jinja2'].get_template(template_name)

            for key, processor in context_processors.items():
                if key not in context:
                    value = await processor(request)
                    context[key] = value

            text = template.render(context)

            return web.Response(
                text=text, status=status, charset='utf-8',
                content_type='text/html'
            )
        return wrapper

    return decorator



from unv.app.core import create_component_settings
from unv.app.helpers import get_project_root

PROJECT_ROOT = get_project_root()

PUBLIC_PRIVATE_FILES_AND_URLS_SCHEMA = {
    "type": "object",
    "properties": {
        "paths": {
            "type": "object",
            "properties": {
                "public": {"type": "string", "required": True},
                "private": {"type": "string", "required": True}
            }
        },
        "urls": {
            "type": "object",
            "properties": {
                "public": {"type": "string", "required": True},
                "private": {"type": "string", "required": True}
            }
        }
    }
}

SCHEMA = {
    "type": "object",
    "properties": {
        "domain": {
            "type": "string",
            "required": "true"
        },
        "protocol": {
            "type": "string",
            "allowed": ["http", "https"],
            "required": "true"
        },
        "host": {
            "type": "string",
            "required": "true"
        },
        "port": {
            "type": "integer",
            "required": "true"
        },
        "autoreload": {
            "type": "boolean",
            "required": "true"
        },
        "static": PUBLIC_PRIVATE_FILES_AND_URLS_SCHEMA,
        "media": PUBLIC_PRIVATE_FILES_AND_URLS_SCHEMA
    }
}

DEFAULTS = {
    'domain': 'app.local',
    'protocol': 'https',
    'host': '0.0.0.0',
    'port': 8000,
    'autoreload': False,
    'static': {
        'paths': {
            'public': str(PROJECT_ROOT / 'static' / 'public'),
            'private': str(PROJECT_ROOT / 'static' / 'private')
        },
        'urls': {
            'public': '/static/public',
            'private': '/static/private'
        }
    },
    'media': {
        'paths': {
            'public': str(PROJECT_ROOT / 'media' / 'public'),
            'private': str(PROJECT_ROOT / 'media' / 'private')
        },
        'urls': {
            'public': '/media/public',
            'private': '/media/private'
        }
    }
}

SETTINGS = create_component_settings('web', DEFAULTS, SCHEMA)
